/* COPYRIGHT NOTICE
Portions Copyright (C) 2017 Scott H. Daggett

Find out more about this add on code at ninjalaunchpad.com/downloads.html.

This program is free software: you can redistribute it and/or modify it under the terms of
the GNU General Public License version 3 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.
If not, see https://www.gnu.org/licenses/gpl-3.0.en.html.

You must include this full COPYRIGHT NOTICE at the beginning of any copies of the file that you redistribute.
*/


#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.Gui.Tools;
// Manually added:
using NinjaTrader.NinjaScript.Indicators;
using System.IO;
#endregion

//This namespace holds Add ons in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.AddOns
{
	public class BzvSupportResistanceCollection : NinjaTrader.NinjaScript.AddOnBase
	{
		#region Variables
		protected DateTime? NO_TIME = null;	// Nullable DateTime, see https://msdn.microsoft.com/en-us/library/b3h38hb0(v=vs.110).aspx
		#endregion
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description = @"Provides a custom collection to hold and manipulate support and resistance lines.";
				Name = "BzvSupportResistanceCollection";
			}
			else if (State == State.Configure)
			{
			}
		}
		
		protected void createSRList()
		{
			SRLines = new List<BzvSRLineV1>();
		}
		
		/// <summary>
		/// Adds the passed in line definition to the list of lines.
		/// Assumes that all passed in lines have valid DateTimes with
		/// an end DateTime? of null representing an undefined date, meaning that
		/// the line has not yet found a termination point. In other words,
		/// this method does not do any error checking.
		/// </summary>
		/// <param name="newLine">A fully-defined line</param>
		public void AddLine(BzvSRLineV1 newLine)
		{
			if (SRLines == null) createSRList();
			SRLines.Add(newLine);
		}
		
		public void RemoveLine(BzvSRLineV1 lineToRemove)
		{
			if (SRLines == null) return;
			SRLines.Remove(lineToRemove);
		}
		
		public void RemoveLineAt(int lineIndexToRemove)
		{
			if (SRLines == null) return;
			SRLines.RemoveAt(lineIndexToRemove);
		}
		
		public BzvSRLineV1 GetSRLine(double nearPrice, DateTime whenTime, BzvUpOrDownV1 whichSide, int levelsAway)
		{
			//---------------------------------------------------------------------------------------------------
			// The idea here is to build a List of lines that meet the active time criteria on the whichSide
			// of nearPrice (including equal), then sort it SO THAT index 0 is level 1, index 1 is level 2 and
			// so on. Then, use (whichSide - 1) as the index to the right answer and return that price.
			// Keep in mind, that there may be more than one active line at the same price, so such
			// "duplicate price" lines should be removed. Also keep in mind that there may not be a line at
			// the requested level, so check for existence and return -1.00 if it doesn't exist.
			//---------------------------------------------------------------------------------------------------

			BzvSRLineV1 srPriceLine = null;	// Use null to denote no result (no matching line found)
			if (SRLines == null) return srPriceLine;
			List<BzvSRLineV1> activeLines = new List<BzvSRLineV1>();
			// Build the array of active lines that meet the criteria:
			foreach(BzvSRLineV1 line in SRLines)
			{
				// First, reject all lines that are not active as of whenTime
				if (line.SRStartTime > whenTime || (line.SREndTime != NO_TIME && line.SREndTime < whenTime))
				{
					continue;
				}
				// If the levelsAway is 1 and the line in question's price matches nearPrice, then jsut return it
				// now and avoid further work...
				if (line.SRPrice == nearPrice && levelsAway == 1)
				{
						srPriceLine = line;	// This is the answer, return it
						return srPriceLine;
						break;					// For safe keeping. Not really needed.
				}
				// Now filter out lines on the wrong side of nearPrice, keeping in mind that nearPrice is still good
				// for levelsAway > 1.
				if (line.SRPrice != nearPrice && ((line.SRPrice < nearPrice && whichSide == BzvUpOrDownV1.Up) || line.SRPrice > nearPrice && whichSide == BzvUpOrDownV1.Down))
				{
					continue;
				}
				// At this point, we have a line that meets our criteria.
				// So build the temporary List as we loop through so we have a collection of all active lines
				// on the right side of nearPrice. Don't add a line whose price is already there.
				// Once we have that List, we can sort it and then get the right level to send back.
				// Keep in mind that there may be NO lines that fit the criteria.
				if (!activeLines.Exists(e => e.SRPrice == line.SRPrice))
				{
					activeLines.Add(line);
				}
			}
			// We've got all the non-duplicate active lines in our temp List.
			// Now sort the active lines list in the right direction and remove duplicates
			Comparison<BzvSRLineV1> compPrice;
			if (whichSide == BzvUpOrDownV1.Up)
			{
            	compPrice = new Comparison<BzvSRLineV1>(BzvSRLineV1.CompareAsc);
			}
			else
			{
				compPrice = new Comparison<BzvSRLineV1>(BzvSRLineV1.CompareDec);
			}
            activeLines.Sort(compPrice);
			
			// Now make sure the requested level is there and return it's price
			if (activeLines.Count >= levelsAway)
			{
				srPriceLine = activeLines[levelsAway-1];
			} // else it is already -1.00
			
			return srPriceLine;
		}
		
		public void LoadLinesFromCsvFile(string filePath)
		{
			LoadLinesFromCsvFile(filePath, (DateTime?)null);
		}

		/// <summary>
		/// Quick and dirty load method. Doesn't have a whole lot of safety checks in place.
		/// Just make sure the data file is good. If any bad data or srtucture exists on a line, it will
		/// just get ignored. If things are bad enough, then we'll just fail the thing and make the
		/// user go fix the file. This should ignore header lines and any bad lines.
		/// </summary>
		/// <param name="filePath">Full path to the the .srlines file that conforms to the format.</param>
		/// <param name="fromDateTime">If not null, then lines that expire prior to this DateTime are not loaded</param>
		public void LoadLinesFromCsvFile(string filePath, DateTime? fromDateTime)
		{
			string currLine;
			string methodName = ".LoadLinesFromCsvFile().";
			
			// See if the thing exists
			if (!File.Exists(filePath))
			{
				Print("Designated srlines file does not exist in " + this.Name + methodName);
				Print("Given file path: " + filePath);
				return;
			}
			try
			{
				using (StreamReader csvReader = new StreamReader(filePath))
				{
				
					currLine = csvReader.ReadLine();
					while (currLine != null && currLine.Contains(","))
					{
						// Inner try/catch block is here so that a line can get ignored on error
						// and then the loop can continue.
						try
						{
							BzvSRLineV1 newLine = BzvSRLineV1.ParseCsvLine(currLine);	// Could throw exception
							// Check to make sure we're not adding a line whose end date is before the cutoff DateTime
							if(fromDateTime == null || newLine.SREndTime == null || newLine.SREndTime >= fromDateTime)
							{
								AddLine(newLine);
							}
							currLine = csvReader.ReadLine();
						}
						catch
						{
							// Just ignore and go to the next line.
							Print("\nLine from data file, " + filePath + ", was ignored. Data line from file:");
							Print(currLine);
							currLine = csvReader.ReadLine();
							continue;
						}
					}
				}
				
			}
			catch (Exception e)
			{
				Print("Excecption thrown in " + this.Name + methodName);
				Print("Exception: " + e.Message);
				return;
			}
		}
		
		/// <summary>
		/// This will overwrite, without warning, the file at the path passed in.
		/// </summary>
		/// <param name="filePath">Full path to the file. use '.srlines' as suffix as a standard. (not enforced)</param>
		public void WriteLinesToCsvFile(string filePath)
		{
			string currLine;
			string[] allLines = new string[SRLines.Count];
			int idx = 0;
			foreach (BzvSRLineV1 line in SRLines)
			{
				currLine = line.ToPersistentCSVString();
				allLines[idx] = currLine;
				idx++;
			}
			try
			{
				System.IO.File.WriteAllLines(filePath, allLines);
			}
			catch
			{
			}
		}
		
		#region Properties
		
		public List<BzvSRLineV1> SRLines
		{ get; set; }

		#endregion
	}

    public class BzvSRLineV1
    {
		#region Instance Variables
		
        private DateTime srStartTime;
		private DateTime? srEndTime;	// Nullable DateTime, see https://msdn.microsoft.com/en-us/library/b3h38hb0(v=vs.110).aspx
		private double srPrice;
		private BzvSRPrimaryTypeV1 srPrimaryType;
		private BzvSRSecondaryTypeV1 srSecondaryType;
		private BzvPersistenceV1 srPersistence;
		
		#endregion
		
		public BzvSRLineV1(DateTime start, DateTime? end, double thePrice, BzvSRPrimaryTypeV1 primType,
					BzvSRSecondaryTypeV1 secType, BzvPersistenceV1 persistence)
		{
			InitMembers(start, end, thePrice, primType, secType, persistence);
		}
		
		public BzvSRLineV1()
		{
		}
		
		public void InitMembers(DateTime start, DateTime? end, double thePrice, BzvSRPrimaryTypeV1 primType,
					BzvSRSecondaryTypeV1 secType, BzvPersistenceV1 persistence)
		{
			srStartTime = start;
			srEndTime = end;
			srPrice = thePrice;
			srPrimaryType = primType;
			srSecondaryType = secType;
			srPersistence = persistence;
		}
		
		/// <summary>
		/// Start Date is in the format: yyyy-mm-ddThh:mm:ss   Example: '2009-06-15T13:45:30.1234' (for en-US, that is).
		/// End Date is in the same format or an empty string.
		/// Price is next.
		/// Then Primary Type is Auction, Range, or Other
		/// Then Secondary Type is High, Low, VPOC, POC
		/// Then Persistence is "Retain" or "Purge"
		/// </summary>
		/// <returns>CSV-style string of the line's data suitable for file storage</returns>
		public string ToPersistentCSVString()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			string csvStr = "";
			string comma = ",";
			sb.Append(srStartTime.ToString("s") + comma);
			sb.Append((srEndTime == null ? "" : srEndTime.Value.ToString("s")) + comma);
			sb.Append(srPrice.ToString() + comma);
			sb.Append(srPrimaryType.ToString() + comma);
			sb.Append(srSecondaryType.ToString() + comma);
			sb.Append(srPersistence.ToString());
			
			return sb.ToString();
		}
		
		public static BzvSRLineV1 ParseCsvLine(string csvLine)
		{
			string[] currFields;
			Char delim = ',';
			BzvSRLineV1 srLine = null;
			string dtFormat = "o";
			// Temp holder variables
			DateTime start;
			DateTime? end;
			double thePrice;
			BzvSRPrimaryTypeV1 primaryType;
			BzvSRSecondaryTypeV1 secondaryType;
			BzvPersistenceV1 persistence;

			currFields = csvLine.Split(delim);
			if (currFields.Length == 6)
			{
				start = DateTime.Parse(currFields[0]);
				if (currFields[1] == null || currFields[1].Trim() == "")
				{
					end = (DateTime?)null;
				}
				else
				{
					end = (DateTime?)DateTime.Parse(currFields[1]);
				}
				thePrice = double.Parse(currFields[2]);
				primaryType = (BzvSRPrimaryTypeV1) Enum.Parse(typeof(BzvSRPrimaryTypeV1), currFields[3]);
				secondaryType = (BzvSRSecondaryTypeV1) Enum.Parse(typeof(BzvSRSecondaryTypeV1), currFields[4]);
				persistence = (BzvPersistenceV1) Enum.Parse(typeof(BzvPersistenceV1), currFields[5]);
			}
			else
			{
				throw new Exception("Passed in line has incorrect number of comma-separated fields.");
			}
			
			srLine = new BzvSRLineV1(start, end, thePrice, primaryType, secondaryType, persistence);
			return srLine;
		}
		
		#region Comparison Methods
		
        public static int CompareAsc(BzvSRLineV1 l1, BzvSRLineV1 l2)
        {
			return l1.SRPrice.CompareTo(l2.SRPrice);
        }
		
        public static int CompareDec(BzvSRLineV1 l1, BzvSRLineV1 l2)
        {
			return -l1.SRPrice.CompareTo(l2.SRPrice);
        }
		
		#endregion
		
		#region Properties
		
		public DateTime SRStartTime
		{
			get { return srStartTime; }
			set { srStartTime = value; }
		}
		
		public DateTime? SREndTime
		{
			get { return srEndTime; }
			set { srEndTime = value; }
		}
		
		public double SRPrice
		{
			get { return srPrice; }
			set { srPrice = value; }
		}
		
		public BzvSRPrimaryTypeV1 SRPrimaryType
		{
			get { return srPrimaryType; }
			set { srPrimaryType = value; }
		}
		
		public BzvSRSecondaryTypeV1 SRSecondaryType
		{
			get { return srSecondaryType; }
			set { srSecondaryType = value; }
		}
		
		public BzvPersistenceV1 SRPersistence
		{
			get { return srPersistence; }
			set { srPersistence = value; }
		}
		
		#endregion
		
    } // end class BzvSRLineV1
	
	#region Enums in Indicators namespace
	
	public enum BzvSRPrimaryTypeV1
	{
		Auction,
		Range,
		Other
	}

	public enum BzvSRSecondaryTypeV1
	{
		High,
		Low,
		VPOC,
		POC,
		Other
	}
	
	public enum BzvUpOrDownV1
	{
		Up,
		Down
	}
	
	public enum BzvPersistenceV1
	{
		Retain,
		Purge
	}
	
	#endregion	

}	// ------------------------End of AddOns namespace

