/* COPYRIGHT NOTICE
Portions Copyright (C) 2017 Scott H. Daggett

Find out more about this indicator at ninjalaunchpad.com/downloads.html.

This program is free software: you can redistribute it and/or modify it under the terms of
the GNU General Public License version 3 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.
If not, see https://www.gnu.org/licenses/gpl-3.0.en.html.

You must include this full COPYRIGHT NOTICE at the beginning of any copies of the file that you redistribute.
*/

#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
// Manually added:
using NinjaTrader.NinjaScript.AddOns;
using System.IO;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class BzvSupportResistanceLinesV1 : Indicator
	{
		#region Variables
		private BzvSupportResistanceCollection srCollection = null;
		private string fullDataPath;
		private List<BzvSRLineV1> nullEndLines = null;
		private const int LINE_WIDTH = 2;
		#endregion

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Draws support and resistance lines on a chart and provides methods for other Indicators to find such lines for analysis purposes. Does not discover support and resistance lines on its own; must be provided in a data file.";
				Name										= "BzvSupportResistanceLinesV1";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= false;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				DataFilePath					= string.Empty;
				DataFileName					= string.Empty;
			}
			else if (State == State.Configure)
			{
				// Validate that we have a good path and file (make sure it exists anyway)
				if (!DataFileName.Trim().Equals(string.Empty) && !DataFilePath.Trim().Equals(string.Empty))
				{
					fullDataPath = DataFilePath.EndsWith(@"\") ? DataFilePath + DataFileName :  DataFilePath + @"\" + DataFileName;
					if (File.Exists(fullDataPath))
					{
						// Create the colleciton for use later...
						srCollection = new BzvSupportResistanceCollection();
					}
				}
				if (srCollection == null)
				{
					// No file at given path or not path given. Let the user know...
					string message = "Indicator, " + this.Name + ", requires an existing data file path and file name to be provided in the properties window.";
					MessageBox.Show(message, "Indicator Setup Error", MessageBoxButton.OK, MessageBoxImage.Exclamation);
				}
			}
		}

		protected override void OnBarUpdate()
		{
			// If no lines collection, then just get out...
			if (srCollection == null) return;
			
			// On the first bar, load the data to make it available to other Indicators
			// (Load it here because we need a DateTime to specify which lines to load.)
			if (CurrentBar == 0)
			{
				try
				{
					DateTime fromTime = Time[0];
					srCollection.LoadLinesFromCsvFile(fullDataPath, fromTime);
				}
				catch
				{
				}
			}

			// On (about) the final bar of the chart (and for each new bar in realtime), draw or update the SR lines
			if (CurrentBar >= Bars.Count - 2)
			{
				drawLines();
			}
		}

		
		protected void drawLines()
		{
			// Plan of attack:
			// We only really want to draw lines on the first call. After that, we just want to update
			// lines that have null end dates (assuming that there are no lines with end dates beyond current time).
			// That means we'll need to keep track of the tags of the drawn lines. Maybe we just use some
			// concatenation of start datetime, price and, uh, hmmm. Let's assume that's good enough.
			// It makes sense that we also have a separate list of lines with null end times.
			// The first time through, build the separate list of lines with null end times. If it doesn't
			// already exist, then assume it's the first time through. If it does exist, then just loop
			// through those.
			if (nullEndLines == null)
			{
				drawLines(srCollection.SRLines);
				nullEndLines = new List<BzvSRLineV1>();
				foreach (BzvSRLineV1 line in srCollection.SRLines)
				{
					if (line.SREndTime == null)
					{
						nullEndLines.Add(line);
					}
				}
			}
			else
			{
				drawLines(nullEndLines);
			}
		}
		
		protected void drawLines(List<BzvSRLineV1> lineList)
		{
			Brush lineBrush;
			string tag;
			DashStyleHelper dash = DashStyleHelper.Solid;
			DateTime endTime;
			foreach(BzvSRLineV1 line in lineList)
			{
				tag = line.SRStartTime.ToString("s") + line.SRPrice.ToString();
				endTime = line.SREndTime == null ? Time[0] : line.SREndTime.Value;
				lineBrush = getLinesBrush(line);
				Draw.Line(this, tag, false, line.SRStartTime, line.SRPrice, endTime, line.SRPrice, lineBrush, dash, LINE_WIDTH, true);
			}
		}
		
		protected Brush getLinesBrush(BzvSRLineV1 line)
		{
			Brush theBrush;
			switch (line.SRSecondaryType)
			{
				case BzvSRSecondaryTypeV1.High:
					theBrush = Brushes.Red;
					break;
				case BzvSRSecondaryTypeV1.Low:
					theBrush = Brushes.Red;
					break;
				case BzvSRSecondaryTypeV1.POC:
					theBrush = Brushes.Blue;
					break;
				case BzvSRSecondaryTypeV1.VPOC:
					theBrush = Brushes.Blue;
					break;
				default:
					theBrush = Brushes.DeepSkyBlue;
					break;
			}
			return theBrush;
		}
		
		public BzvSRLineV1 GetSRLine(double nearPrice, DateTime whenTime, BzvUpOrDownV1 whichSide, int levelsAway)
		{
			if (srCollection == null) return (BzvSRLineV1)null;
			return srCollection.GetSRLine(nearPrice, whenTime, whichSide, levelsAway);
		}
			
		#region Properties
		
		[NinjaScriptProperty]
		[Display(Name="DataFilePath", Description="Path to the folder holding the SR Lines data file.", Order=1, GroupName="Parameters")]
		public string DataFilePath
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="DataFileName", Description="Name of the SR Lines data file. (.srlines file)", Order=2, GroupName="Parameters")]
		public string DataFileName
		{ get; set; }
		
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private BzvSupportResistanceLinesV1[] cacheBzvSupportResistanceLinesV1;
		public BzvSupportResistanceLinesV1 BzvSupportResistanceLinesV1(string dataFilePath, string dataFileName)
		{
			return BzvSupportResistanceLinesV1(Input, dataFilePath, dataFileName);
		}

		public BzvSupportResistanceLinesV1 BzvSupportResistanceLinesV1(ISeries<double> input, string dataFilePath, string dataFileName)
		{
			if (cacheBzvSupportResistanceLinesV1 != null)
				for (int idx = 0; idx < cacheBzvSupportResistanceLinesV1.Length; idx++)
					if (cacheBzvSupportResistanceLinesV1[idx] != null && cacheBzvSupportResistanceLinesV1[idx].DataFilePath == dataFilePath && cacheBzvSupportResistanceLinesV1[idx].DataFileName == dataFileName && cacheBzvSupportResistanceLinesV1[idx].EqualsInput(input))
						return cacheBzvSupportResistanceLinesV1[idx];
			return CacheIndicator<BzvSupportResistanceLinesV1>(new BzvSupportResistanceLinesV1(){ DataFilePath = dataFilePath, DataFileName = dataFileName }, input, ref cacheBzvSupportResistanceLinesV1);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.BzvSupportResistanceLinesV1 BzvSupportResistanceLinesV1(string dataFilePath, string dataFileName)
		{
			return indicator.BzvSupportResistanceLinesV1(Input, dataFilePath, dataFileName);
		}

		public Indicators.BzvSupportResistanceLinesV1 BzvSupportResistanceLinesV1(ISeries<double> input , string dataFilePath, string dataFileName)
		{
			return indicator.BzvSupportResistanceLinesV1(input, dataFilePath, dataFileName);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.BzvSupportResistanceLinesV1 BzvSupportResistanceLinesV1(string dataFilePath, string dataFileName)
		{
			return indicator.BzvSupportResistanceLinesV1(Input, dataFilePath, dataFileName);
		}

		public Indicators.BzvSupportResistanceLinesV1 BzvSupportResistanceLinesV1(ISeries<double> input , string dataFilePath, string dataFileName)
		{
			return indicator.BzvSupportResistanceLinesV1(input, dataFilePath, dataFileName);
		}
	}
}

#endregion
